//-----------------------------------------------------------------------------
//  
// linux/drivers/video/epson/s1d13xxxfb_indirect.c -- Indirect interface 
// functions for Epson S1D13xxx controller frame buffer drivers.
// 
// Copyright(c) Seiko Epson Corporation 2000-2005.
// All rights reserved.
//
// This file is subject to the terms and conditions of the GNU General Public
// License. See the file COPYING in the main directory of this archive for
// more details.
//
//---------------------------------------------------------------------------- 
//---------------------------------------------------------------------------- 
//
// WARNING: This file uses a chip specific include file. The include file 
// contains all hardware information needed to generate proper timings, color
// depth, screen resolution etc.
//
// This driver supports one of the following Epson CRT/TV/LCD controllers:
//
// S1D13717
//
//-----------------------------------------------------------------------------


#include <linux/smp_lock.h>

#include "s1d13xxxfb_indirect.h"

#ifdef CONFIG_FBCON_EPSON_INDIRECT

//-----------------------------------------------------------------------------
// Local globals
//----------------------------------------------------------------------------- 
static S1D13XXXFB_IND_STRUCT S1DIndInfo;			// indirect pointers structure.

//-----------------------------------------------------------------------------
// Private Function Prototypes 
//-----------------------------------------------------------------------------
static void             IndirectWriteDisplayAddress( u32 Offset );

//---------------------------------------------------------------------------
// INDIRECT FUNCTION: s1d13xxxfb_IndirectInit() - Initialize to communicate using indirect interface
//---------------------------------------------------------------------------
void s1d13xxxfb_IndirectInit( u32 RegAddr )
{
	S1DIndInfo.pCmd			= (u16*) (RegAddr+0);
	S1DIndInfo.pData16		= (u16*) (RegAddr+2);
}

//---------------------------------------------------------------------------
// INDIRECT FUNCTION: s1d13xxxfb_IndirectWriteReg()
// WARNING: DO NOT USE ODD ADDRESSES FOR 16-bit or 32bit ACCESSES!
//---------------------------------------------------------------------------
void s1d13xxxfb_IndirectWriteReg( u16 Index, u16 Value )
{
	lock_kernel();

	*S1DIndInfo.pCmd = (u16)Index;
	*S1DIndInfo.pData16 = (u16)Value;

	unlock_kernel();
}

//---------------------------------------------------------------------------
// INDIRECT FUNCTION: s1d13xxxfb_IndirectReadReg()
// WARNING: DO NOT USE ODD ADDRESSES FOR 16-bit or 32bit ACCESSES!
//---------------------------------------------------------------------------
u16 s1d13xxxfb_IndirectReadReg( u16 Index )
{
	u16 Value = 0;

	lock_kernel();

	*S1DIndInfo.pCmd = Index;
	Value = *S1DIndInfo.pData16;

	unlock_kernel();

	return Value;
}

//---------------------------------------------------------------------------
// INDIRECT FUNCTION: s1d13xxxfb_IndirectWriteData()
// WARNING: DO NOT USE ODD ADDRESSES FOR 16-bit or 32bit ACCESSES!
//---------------------------------------------------------------------------
void s1d13xxxfb_IndirectWriteData( u32 Offset, const void* pData, u32 Count )
{
	if ( Count && pData ) {
		lock_kernel();
		{
			const u16* pRead = pData;
			IndirectWriteDisplayAddress( Offset );
			do {
				*S1DIndInfo.pData16 = *pRead++;
			} while ( --Count );
		}
		unlock_kernel();
	}
}

//---------------------------------------------------------------------------
// PRIVATE FUNCTION: IndirectWriteDisplayAddress()
// Sets up display address offset and enforces auto-increment mode.
// This function MUST be called within a process exclusive block!
//---------------------------------------------------------------------------
static void IndirectWriteDisplayAddress( u32 Offset )
{
	*S1DIndInfo.pCmd = REG0026_IAUTOINC;
	*S1DIndInfo.pData16 = 0x0000;				// Select byte/word auto increment

	*S1DIndInfo.pCmd = REG0022_IMEMADDR1;
	*S1DIndInfo.pData16 = (u16)Offset;
	*S1DIndInfo.pCmd = REG0024_IMEMADDR2;
	*S1DIndInfo.pData16 = (u16)(Offset >> 16);

	*S1DIndInfo.pCmd = REG0028_IMEMPORT;
}

#endif  //CONFIG_FBCON_EPSON_INDIRECT
