/*
 * AT91 GPIO 1-Wire master device driver
 *
 * Copyright (C) 2007 taskit GmbH
 */

#include <linux/delay.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <asm/arch/gpio.h>
#include <asm/arch/board.h>
#include "../w1.h"
#include "../w1_int.h"

static void at91_gpio_w1_set_pin(void *data, u8 val)
{
	at91_set_gpio_value(((struct at91_gpio_w1_data*)data)->pin, val);
}

static u8 at91_gpio_w1_read_pin(void *data)
{
	return at91_get_gpio_value(((struct at91_gpio_w1_data*)data)->pin);
}


int at91_gpio_w1_probe(struct platform_device *pdev)
{
	int pin;
	struct w1_bus_master *master = kzalloc(sizeof(struct w1_bus_master), GFP_KERNEL);
	master->read_bit = &at91_gpio_w1_read_pin;
	master->write_bit = &at91_gpio_w1_set_pin;
	master->data = pdev->dev.platform_data;
	platform_set_drvdata(pdev, master);

	pin=((struct at91_gpio_w1_data*)master->data)->pin;
	
	at91_set_gpio_output(pin, 1);
	at91_set_multi_drive(pin, 1);

	return w1_add_master_device(master);
}

int at91_gpio_w1_remove(struct platform_device *pdev)
{
	struct w1_bus_master *master = platform_get_drvdata(pdev);
	w1_remove_master_device(master);
	kfree(master);
	return 0;
}

static struct platform_driver at91_gpio_w1_driver = {
	.probe  = at91_gpio_w1_probe,
	.remove = at91_gpio_w1_remove,
	.driver = {
		.name = "w1_at91gpio",
	},
};

int at91_gpio_w1_init(void)
{
	return platform_driver_register(&at91_gpio_w1_driver);
}

void at91_gpio_w1_exit(void)
{
	platform_driver_unregister(&at91_gpio_w1_driver);
}

module_init(at91_gpio_w1_init);
module_exit(at91_gpio_w1_exit);

MODULE_AUTHOR("Christian Glindkamp");
MODULE_DESCRIPTION("GPIO 1-Wire master device driver for AT91 based boards");
MODULE_LICENSE("GPL v2");
